import scrapy
from scraper_hj3415.nfscraper.nfs import items

# 여러 잡다한 기호를 없애거나 교체하는 람다함수
def cleaning(s):
    if s is None or s == 'N/A':
        return None
    return (
        s.strip().replace('\t', '').replace('\r', '').replace('\n', '').replace(',', '')
        .replace('원', '').replace('주', '').replace('억', '00000000').replace('%', '')
    )

# CSS Path 생성 함수
def css_path(i, j=None):
    if j:
        return f'#pArea>div.wrapper-table>div>table>tr:nth-child({i})>td>dl>dt:nth-child({j})>'
    return f'#cTB11>tbody>tr:nth-child({i})>'

# None일 때 빈 문자열 반환
def str_or_blank(s):
    return '' if s is None else str(s)

class C101Spider(scrapy.Spider):
    name = 'c101'
    allowed_domains = ['navercomp.wisereport.co.kr']  # https 주소

    def __init__(self, *args, **kwargs):
        super(C101Spider, self).__init__(*args, **kwargs)
        self.codes = kwargs.get("codes", [])

    def start_requests(self):
        total_count = len(self.codes)
        self.logger.info(f'Start scraping {self.name}, {total_count} items...')
        for one_code in self.codes:
            url = f'https://navercomp.wisereport.co.kr/v2/company/c1010001.aspx?cmp_cd={one_code}'
            yield scrapy.Request(url=url, callback=self.parse_c101, cb_kwargs=dict(code=one_code))

    def parse_c101(self, response, code):
        self.logger.info(f'Parsing {self.name} for {code}')
        item = items.C101items()

        try:
            item['date'] = (response.xpath('//*[@id="wrapper"]/div[1]/div[1]/div[1]/dl/dd[2]/p/text()').get()
                            .replace('[기준:', '').replace(']', ''))
        except AttributeError:
            self.logger.error(f'ERROR: Page not found for {code}')
            return

        item.update({
            'code': cleaning(response.css(css_path(1, 1) + 'b::text').get()),
            'page': 'c101',
            '종목명': response.css(css_path(1, 1) + 'span::text').get(),
            '업종': response.css(css_path(1, 4).rstrip('>') + '::text').get().replace('WICS : ', ''),
            'EPS': cleaning(response.css(css_path(3, 1) + 'b::text').get()),
            'BPS': cleaning(response.css(css_path(3, 2) + 'b::text').get()),
            'PER': cleaning(response.css(css_path(3, 3) + 'b::text').get()),
            '업종PER': cleaning(response.css(css_path(3, 4) + 'b::text').get()),
            'PBR': cleaning(response.css(css_path(3, 5) + 'b::text').get()),
            '배당수익률': cleaning(response.css(css_path(3, 6) + 'b::text').get()),
            '주가': cleaning(response.css(css_path(1) + 'td>strong::text').get()),
        })

        # 52주 최고/최저, 거래량/거래대금, 발행주식/유동비율 등 데이터 처리
        try:
            최고최저52주 = response.css(css_path(2) + 'td::text').get().split('/')
            item['최고52주'], item['최저52주'] = map(cleaning, 최고최저52주)

            거래량거래대금 = response.css(css_path(4) + 'td::text').get().split('/')
            item['거래량'], item['거래대금'] = map(cleaning, 거래량거래대금)

            발행주식유동비율 = response.css(css_path(7) + 'td::text').get().split('/')
            item['발행주식'], item['유동비율'] = map(cleaning, 발행주식유동비율)
        except (AttributeError, ValueError):
            self.logger.error(f'Parsing error for {code}')

        item.update({
            '시가총액': cleaning(response.css(css_path(5) + 'td::text').get()),
            '베타52주': cleaning(response.css(css_path(6) + 'td::text').get()),
            '외국인지분율': cleaning(response.css(css_path(8) + 'td::text').get()),
            'intro1': str_or_blank(response.xpath('//*[@id="wrapper"]/div[5]/div[2]/ul/li[1]/text()').get()),
            'intro2': str_or_blank(response.xpath('//*[@id="wrapper"]/div[5]/div[2]/ul/li[2]/text()').get()),
            'intro3': str_or_blank(response.xpath('//*[@id="wrapper"]/div[5]/div[2]/ul/li[3]/text()').get()),
        })

        self.logger.info(item)
        yield item


# Define here the models for your scraped items
#
# See documentation in:
# https://docs.scrapy.org/en/latest/topics/items.html

import scrapy


class C101items(scrapy.Item):
    code = scrapy.Field()
    page = scrapy.Field()

    date = scrapy.Field()
    종목명 = scrapy.Field()
    업종 = scrapy.Field()
    EPS = scrapy.Field()    # 주당순이익 EPS*PER = 주가
    BPS = scrapy.Field()    # 주당장부가치
    PER = scrapy.Field()    # 기업가치가 순이익에 몇배에 거래되는가
    업종PER = scrapy.Field()
    PBR = scrapy.Field()
    배당수익률 = scrapy.Field()

    주가 = scrapy.Field()
    최고52주 = scrapy.Field()
    최저52주 = scrapy.Field()
    거래량 = scrapy.Field()
    거래대금 = scrapy.Field()
    시가총액 = scrapy.Field()
    베타52주 = scrapy.Field()
    발행주식 = scrapy.Field()
    유동비율 = scrapy.Field()
    외국인지분율 = scrapy.Field()

    #기업개요 파트
    intro1 = scrapy.Field()
    intro2 = scrapy.Field()
    intro3 = scrapy.Field()


class C103468items(scrapy.Item):
    code = scrapy.Field()
    page = scrapy.Field()
    df = scrapy.Field()



# Define here the models for your scraped items
#
# See documentation in:
# https://docs.scrapy.org/en/latest/topics/items.html

import scrapy


class MisItems(scrapy.Item):
    title = scrapy.Field()
    date = scrapy.Field()
    value = scrapy.Field()



import sys
from itemadapter import ItemAdapter
from pymongo.errors import BulkWriteError

from db_hj3415 import mymongo, myredis
from utils_hj3415 import setup_logger

mylogger = setup_logger(__name__,'WARNING')


class ValidationPipeline:
    def process_item(self, item, spider):
        print(f"\tIn ValidationPipeline.. code : {item['code']} / page : {item['page']}")
        if spider.name == 'c101':
            mylogger.info(f"Raw data - EPS:{item['EPS']} BPS:{item['BPS']} PER:{item['PER']} PBR:{item['PBR']}")
            if mymongo.Base.is_there(db=item['code'], table='c104q'):
                c104q = mymongo.C104(item['code'], 'c104q')
                mylogger.debug(f"EPS : {c104q.find('EPS', remove_yoy=True)[1]}")
                mylogger.debug(f"BPS : {c104q.find('BPS', remove_yoy=True)[1]}")
                d, cal_eps = c104q.sum_recent_4q('EPS')  # 최근 4분기 eps값을 더한다.
                d, cal_bps = c104q.latest_value('BPS')  # 마지막 분기 bps값을 찾는다.

                # per, pbr을 구하는 람다함수
                cal_ratio = (lambda eps_bps, pprice:
                             None if eps_bps is None or eps_bps == 0 else round(int(pprice) / int(eps_bps), 2))
                try:
                    cal_per = cal_ratio(cal_eps, item['주가'])
                    cal_pbr = cal_ratio(cal_bps, item['주가'])
                except ValueError:
                    mylogger.info("유효하지 않은 c104q 데이터로 인해 별도 계산 없이 c101 데이터를 사용합니다..")
                else:
                    mylogger.info(f"Calc data - EPS:{cal_eps} BPS:{cal_bps} PER:{cal_per} PBR:{cal_pbr}")
                    item['EPS'], item['BPS'], item['PER'], item['PBR'] = cal_eps, cal_bps, cal_per, cal_pbr
            else:
                mylogger.info("c104q 데이터가 없어서 별도 계산 없이 c101 데이터를 사용합니다..")
        elif 'c103' in spider.name:
            pass
        elif 'c104' in spider.name:
            pass
        elif spider.name == 'c106':
            pass
        elif spider.name == 'c108':
            pass
        return item


class RedisPipeline:
    def process_item(self, item, spider):
        print(f"\tIn RedisPipeline.. code : {item['code']} / page : {item['page']}")
        if spider.name == 'c101':
            pattern = item['code'] + '.c101*'
        elif 'c103' in spider.name:
            pattern = item['code'] + '.c103*'
        elif spider.name == 'c104y':
            pattern = item['code'] + '.c104y*'
        elif spider.name == 'c104q':
            pattern = item['code'] + '.c104q*'
        elif spider.name == 'c106':
            pattern = item['code'] + '.c106*'
        elif spider.name == 'c108':
            pattern = item['code'] + '.c108*'
        else:
            raise Exception
        print(f"\t\tDelete redis data has a pattern '{pattern}'")
        myredis.Base.delete_all_with_pattern(pattern)
        return item


class MongoPipeline:
    def process_item(self, item, spider):
        print(f"\tIn MongoPipeline.. code : {item['code']} / page : {item['page']}")
        if spider.name == 'c101':
            # print(item)
            mymongo.C101.save(item['code'], ItemAdapter(item).asdict())
        elif 'c103' in spider.name:
            # pprint.pprint(item['df'])
            mymongo.C103.save(item['code'], item['page'], item['df'])
        elif 'c104' in spider.name:
            # pprint.pprint(item['df'])
            try:
                mymongo.C104.save(item['code'], item['page'], item['df'])
            except BulkWriteError as e:
                err_str = f"{item['code']} / {item['page']} 서버 저장에 문제가 있습니다.({str(e)[:60]}..)"
                print('\t\t' + err_str, file=sys.stderr)
                mymongo.Logs.save('cli','ERROR', f"{item['code']} / Error on nfs - pipeline : {str(e)}")
        elif spider.name == 'c106':
            # pprint.pprint(item['df'])
            mymongo.C106.save(item['code'], item['page'], item['df'])
        elif spider.name == 'c108':
            # pprint.pprint(item['df'].to_dict('records'))
            mymongo.C108.save(item['code'], item['df'])
        return item



import scrapy
from scraper_hj3415.miscraper.mis import items


class MiSpider(scrapy.Spider):
    name = 'mi'
    allowed_domains = ['finance.naver.com']

    def start_requests(self):
        urls = {
            'aud': [self.parse_aud, 'https://finance.naver.com/marketindex/worldDailyQuote.nhn?fdtc=4&marketindexCd=FX_USDAUD'],
            'chf': [self.parse_chf, 'https://finance.naver.com/marketindex/worldDailyQuote.nhn?fdtc=4&marketindexCd=FX_USDCHF'],
            'gbond3y': [self.parse_gbond3y, 'https://finance.naver.com/marketindex/interestDailyQuote.nhn?marketindexCd=IRR_GOVT03Y'],
            'gold': [self.parse_gold, 'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=CMDT_GC&fdtc=2'],
            'silver': [self.parse_silver, 'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=CMDT_SI&fdtc=2'],
            'usdidx': [self.parse_usdidx, 'https://finance.naver.com/marketindex/worldExchangeDetail.nhn?marketindexCd=FX_USDX'],
            'usdkrw': [self.parse_usdkrw, 'https://finance.naver.com/marketindex/exchangeDailyQuote.nhn?marketindexCd=FX_USDKRW'],
            'wti': [self.parse_wti, 'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=OIL_CL&fdtc=2'],
            'kosdaq': [self.parse_kosdaq, 'https://finance.naver.com/sise/sise_index_day.nhn?code=KOSDAQ'],
            'kospi': [self.parse_kospi, 'https://finance.naver.com/sise/sise_index_day.nhn?code=KOSPI'],
            'sp500': [self.parse_sp500, 'https://finance.naver.com/world/sise.nhn?symbol=SPI@SPX'],
        }
        print(f'Start scraping Market Index ...')
        for title, (func, url) in urls.items():
            yield scrapy.Request(url=url, callback=func, cb_kwargs=dict(title=title))

    def parse_sp500(self, response, title):
        print(f'<<< Parsing {title}...')
        item = items.MisItems()
        # print(response.text)

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = response.xpath(f'//*[@id="dayTable"]/tbody/tr[{r}]/td[1]/text()').get()
            item['value'] = (response.xpath(f'//*[@id="dayTable"]/tbody/tr[{r}]/td[2]/span/text()')
                             .get().replace(',', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_kospi(self, response, title):
        print(f'<<< Parsing {title}...')
        item = items.MisItems()
        # print(response.text)

        for i in range(3, 6):
            item['title'] = title
            item['date'] = response.xpath(f'/html/body/div/table[1]/tr[{i}]/td[1]/text()').get()
            item['value'] = response.xpath(f'/html/body/div/table[1]/tr[{i}]/td[2]/text()').get().replace(',', '')
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_kosdaq(self, response, title):
        print(f'<<< Parsing {title}...')
        item = items.MisItems()
        # print(response.text)

        for r in range(3, 6):
            item['title'] = title
            item['date'] = response.xpath(f'/html/body/div/table[1]/tr[{r}]/td[1]/text()').get()
            item['value'] = response.xpath(f'/html/body/div/table[1]/tr[{r}]/td[2]/text()').get().replace(',', '')
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_wti(self, response, title):
        print(f'<<< Parsing {title}...')
        item = items.MisItems()
        # print(response.text)

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_usdkrw(self, response, title):
        print(f'<<< Parsing usd/krw ...')
        item = items.MisItems()
        # print(response.text)

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[1]/text()').get()
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[2]/text()')
                             .get().replace(',', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_usdidx(self, response, title):
        print(f'<<< Parsing dollar index ...')
        item = items.MisItems()
        # print(response.text)

        # 최근 데이터를 스크랩한다.
        # date - //*[@id="content"]/div[1]/div[2]/span[1]
        # value - //*[@id="content"]/div[1]/div[1]/p[1]/em
        value = []
        for span in response.xpath(f'//*[@id="content"]/div[1]/div[1]/p[1]/em/span'):
            value.append(span.xpath('text()').get())

        item['title'] = title
        item['date'] = response.xpath('//*[@id="content"]/div[1]/div[2]/span[1]/text()').get()
        item['value'] = ''.join(value)

        print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
        yield item

    def parse_silver(self, response, title):
        print(f'<<< Parsing {title}...')
        # print(response.text)
        item = items.MisItems()

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_gold(self, response, title):
        print(f'<<< Parsing {title}...')
        # print(response.text)
        item = items.MisItems()

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_gbond3y(self, response, title):
        print(f'<<< Parsing {title}...')
        # print(response.text)
        item = items.MisItems()

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = (response.css(f'body > div > table > tbody > tr:nth-child({r}) > td.date::text')
                            .extract()[0].replace('\n', '').replace('\t', ''))
            item['value'] = (response.css(f'body > div > table > tbody > tr:nth-child({r}) > td:nth-child(2)::text')
                .extract()[0])
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_chf(self, response, title):
        print(f'<<< Parsing usd/{title}...')
        # print(response.text)
        item = items.MisItems()

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item

    def parse_aud(self, response, title):
        print(f'<<< Parsing usd/{title}...')
        # print(response.text)
        item = items.MisItems()

        for r in range(3, 0, -1):
            item['title'] = title
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{r}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item








import argparse
import random

from db_hj3415 import mymongo
from utils_hj3415 import tools
from scraper_hj3415.nfscraper.run import c101, c106, c103y, c103q, c104y, c104q, c108, all_spider
from scraper_hj3415.krx300 import krx300 as krx
from scraper_hj3415.miscraper.run import mi, mihistory


def nfs_manager():

    spiders = {
        'c101': c101,
        'c106': c106,
        'c103y': c103y,
        'c103q': c103q,
        'c104y': c104y,
        'c104q': c104q,
        'c108': c108,
        'all_spider': all_spider
    }

    parser = argparse.ArgumentParser(description="NF Scraper Command Line Interface")
    subparsers = parser.add_subparsers(dest='spider', help='사용할 스파이더를 선택하세요.', required=True)

    # 각 스파이더에 대해 서브 파서 설정
    for spider_name, spider_func in spiders.items():
        spider_parser = subparsers.add_parser(spider_name, help=f"{spider_name} 스파이더 실행")
        spider_parser.add_argument('targets', nargs='*', type=str, help="대상 종목 코드를 입력하세요. 'all'을 입력하면 전체 종목을 대상으로 합니다.")

    # 명령줄 인자 파싱
    args = parser.parse_args()

    selected_spider = spiders.get(args.spider)
    if not selected_spider:
        print(f"The spider should be in {list(spiders.keys())}")
        return

    # 전체 종목을 대상으로 할 경우 처리
    if len(args.targets) == 1 and args.targets[0] == 'all':
        all_codes = krx.get_codes()
        random.shuffle(all_codes)
        selected_spider(*all_codes)  # 스파이더 실행
        mymongo.Logs.save('cli','INFO', f"run >> nfs {selected_spider.__name__} all / {len(all_codes)} codes")
    else:
        # 입력된 종목 코드 유효성 검사
        invalid_codes = [code for code in args.targets if not tools.is_6digit(code)]
        if invalid_codes:
            print(f"다음 종목 코드의 형식이 잘못되었습니다: {', '.join(invalid_codes)}")
            return
        selected_spider(*args.targets)  # 스파이더 실행
        mymongo.Logs.save('cli','INFO', f"run >> nfs {selected_spider.__name__} {args.targets}")

def mis_manager():
    parser = argparse.ArgumentParser(description="Market Index Scraper")
    subparsers = parser.add_subparsers(dest='command', help='Available commands', required=True)

    # 'mi' 명령어 서브파서
    parser_mi = subparsers.add_parser('mi', help='오늘의 Market Index를 저장합니다.')

    # 'mihistory' 명령어 서브파서
    parser_mihistory = subparsers.add_parser('mihistory', help='과거 Market Index를 저장합니다.')
    parser_mihistory.add_argument('--years', type=int, default=3, help='저장할 과거 데이터의 연도 수 (기본값: 3년)')

    args = parser.parse_args()

    if args.command == 'mi':
        mymongo.Logs.save('cli','INFO', 'run >> mis mi')
        mi()
    elif args.command == 'mihistory':
        mymongo.Logs.save('cli','INFO', f'run >> mis mihistory --years {args.years}')
        mihistory(args.years)

def krx_manager():
    parser = argparse.ArgumentParser(description="Krx300 Manager")
    subparsers = parser.add_subparsers(dest='command', help='Available commands', required=True)

    # 'sync' 명령어 서브파서
    parser_sync = subparsers.add_parser('sync', help='몽고db와 krx300의 싱크를 맞춥니다.')

    args = parser.parse_args()

    if args.command == 'sync':
        mymongo.Logs.save('cli','INFO', 'run >> krx sync')
        krx.sync_with_mongo()



import time
import scrapy
import pandas as pd
import selenium.common.exceptions
from scrapy.selector import Selector
from io import StringIO

from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By

from scraper_hj3415.nfscraper.nfs import items

# 분기와 년도 2페이지를 스크랩함.


class C106Spider(scrapy.Spider):
    name = 'c106'
    allowed_domains = ['navercomp.wisereport.co.kr']

    custom_settings = {
        'AUTOTHROTTLE_ENABLED': True,
        'DOWNLOAD_DELAY': 1,
        'CONCURRENT_REQUESTS': 1,
        'CONCURRENT_REQUESTS_PER_DOMAIN': 1,
        'CONCURRENT_REQUESTS_PER_IP': 1,
    }

    def __init__(self, *args, **kwargs):
        super(C106Spider, self).__init__(*args, **kwargs)
        self.codes = kwargs.get("codes", [])
        self.webdriver = kwargs.get("webdriver", None)

    def start_requests(self):
        total_count = len(self.codes)
        self.logger.info(f'entire codes list - {self.codes}')
        for i, one_code in enumerate(self.codes):
            # C106의 컬럼명을 얻기 위한 주소
            yield scrapy.Request(
                url=f'https://navercomp.wisereport.co.kr/v2/company/c1060001.aspx?cmp_cd={one_code}',
                callback=self.parse_c106_col,
                cb_kwargs=dict(code=one_code)
            )

    def parse_c106_col(self, response, code):
        print(f'<<< Parsing C106 columns ...{code}')
        self.webdriver.get(response.url)
        self.webdriver.implicitly_wait(10)
        WebDriverWait(self.webdriver, 10).until(
            EC.visibility_of_all_elements_located((By.ID, "cTB611_h"))
        )
        try:
            html = Selector(text=self.webdriver.page_source)
        except selenium.common.exceptions.UnexpectedAlertPresentException:
            self.logger.warning("Parsing error ... maybe 올바른 종목이 아닙니다.")
        else:
            # 컬럼명을 얻어 다음 request에 실어 보낸다.
            cols = []
            for i in range(1, 7):
                title_elements = html.xpath(f'//*[@id="cTB611_h"]/thead/tr/th[{i}]/text()').getall()
                # print(title_elements)
                try:
                    title_name = title_elements[0].strip().replace('.','')
                except IndexError:
                    raise Exception(f"IndexError: {title_elements}")
                if title_name == '항목':
                    cols.append('항목')
                    continue
                try:
                    title_code = title_elements[1].strip()
                except IndexError:
                    title_code = ''
                # 인덱스에 공칸일 경우 데이터베이스 저장시 에러가 발생하기 때문에 추가한 코드
                if title_name == '':
                    title_name = 'Unnamed'
                cols.append(title_name + '/' + title_code)
            print(f'{self.name} column names - {code} >>>> {cols}')

            titles = ['y', 'q']     # pipeline에서 테이블명으로 됨
            for title in titles:
                # C106의 내부의 iframe주소, 분기와 연간 2개임
                # reference from https://docs.scrapy.org/en/latest/topics/request-response.html (request 연쇄보내기)
                # ex - https://navercomp.wisereport.co.kr/company/cF6002.aspx?cmp_cd=005930&finGubun=MAIN&cmp_cd1=&cmp_cd2=&cmp_cd3=&cmp_cd4=&sec_cd=G453010&frq=Y
                yield scrapy.Request(
                    url=f'https://navercomp.wisereport.co.kr/company/cF6002.aspx?cmp_cd={code}'
                        f'&finGubun=MAIN&cmp_cd1=&cmp_cd2=&cmp_cd3=&cmp_cd4=&sec_cd=G453010&frq={title.upper()}',
                    callback=self.parse_c106_iframe,
                    cb_kwargs=dict(code=code, cols=cols, title=title)
                )

    def parse_c106_iframe(self, response, code, cols, title):
        time.sleep(2)
        print(f'<<< Parsing C106 iframes ...{code}')
        page = ''.join(['c106', title])
        print(f"Making dataframe - {code} / {page}..")
        df = C106Spider.get_df_from_html(response.text, cols)
        df['항목'] = (df['항목']
                    .str.replace(r'\(억\)', '', regex=True)
                    .str.replace(r'\(원\)', '', regex=True)
                    .str.replace(r'\(억원\)', '', regex=True)
                    .str.replace(r'\(%\)', '', regex=True))
        # print(df)
        # make item to yield
        item = items.C103468items()
        item['code'] = code
        item['page'] = page
        item['df'] = df
        yield item

    @staticmethod
    def get_df_from_html(html, cols):
        # 전체 html source에서 table 부위만 추출하여 데이터프레임으로 변환
        html_io = StringIO(html)
        df = pd.read_html(html_io)[0]
        # 인덱스 추가
        df.columns = cols
        df.dropna(how='all', inplace=True)
        return df




import time
import scrapy
import random
from scrapy.selector import Selector

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By

from scraper_hj3415.miscraper.mis import items

# 2초 이내는 데이터 캡쳐가 올바르게 안됨
rnd_wait = [3, 4, 5]


class MIHistory(scrapy.Spider):
    name = 'mihistory'
    allowed_domains = ['finance.naver.com']

    # 버튼 클릭시 페이지가 변하는 것이 일관되지 않는것을 해결하기 위한 세팅. 챗gpt 검색
    custom_settings = {
        'AUTOTHROTTLE_ENABLED': True,
        'DOWNLOAD_DELAY': 1,
        'CONCURRENT_REQUESTS': 1,
        'CONCURRENT_REQUESTS_PER_DOMAIN': 1,
        'CONCURRENT_REQUESTS_PER_IP': 1,
    }

    def __init__(self, *args, **kwargs):
        super(MIHistory, self).__init__(*args, **kwargs)
        self.webdriver = kwargs.get("webdriver", None)
        # 과거 몇년치 데이터를 모을 것인가
        self.years = kwargs.get("years", 1)

        # 대략1년전 kospi, kosdaq -> 42, gbond3y -> 38, s&p -> 27, usdkrw -> 26, wti -> 38, gold -> 38, audchf -> 46
        self.last_page_kospi_kosdaq = int(42 * self.years)
        self.last_page_3bond3y = int(38 * self.years)
        self.last_page_sp500 = int(27 * self.years)
        self.last_page_usdkrw = int(26 * self.years)
        self.last_page_wti = int(38 * self.years)
        self.last_page_gold = int(38 * self.years)
        self.last_page_silver = int(38 * self.years)
        self.last_page_audchf = int(46 * self.years)

    def start_requests(self):
        urls = {
            self.parse_kospi: [
                f'https://finance.naver.com/sise/sise_index_day.nhn?code=KOSPI&page={self.last_page_kospi_kosdaq}',
                self.last_page_kospi_kosdaq],
            self.parse_kosdaq: [
                f'https://finance.naver.com/sise/sise_index_day.nhn?code=KOSDAQ&page={self.last_page_kospi_kosdaq}',
                self.last_page_kospi_kosdaq],
            self.parse_gbond3y: [
                f'https://finance.naver.com/marketindex/interestDailyQuote.nhn?marketindexCd=IRR_GOVT03Y&page={self.last_page_3bond3y}',
                self.last_page_3bond3y],
            self.parse_aud: [
                f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?fdtc=4&marketindexCd=FX_USDAUD',
                self.last_page_audchf],
            self.parse_chf: [
                f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?fdtc=4&marketindexCd=FX_USDCHF',
                self.last_page_audchf],
            self.parse_usdkrw: [
                f'https://finance.naver.com/marketindex/exchangeDailyQuote.nhn?marketindexCd=FX_USDKRW&page={self.last_page_usdkrw}',
                self.last_page_usdkrw],
            self.parse_wti: [
                f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=OIL_CL&fdtc=2&page={self.last_page_wti}',
                self.last_page_wti],
            self.parse_gold: [
                f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=CMDT_GC&fdtc=2&page={self.last_page_gold}',
                self.last_page_gold],
            self.parse_silver: [
                f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=CMDT_SI&fdtc=2&page={self.last_page_silver}',
                self.last_page_silver],
            self.parse_sp500: [
                'https://finance.naver.com/world/sise.nhn?symbol=SPI@SPX',
                self.last_page_sp500],
        }
        # reference from https://docs.scrapy.org/en/latest/topics/request-response.html
        print(f'Start scraping Market Index history...{self.years} year..')

        for func, (url, page) in urls.items():
            yield scrapy.Request(url=url, callback=func, cb_kwargs=dict(page=page))

    def parse_kospi(self, response, page):
        print(f"Parsing ...kospi {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in [12, 11, 10, 5, 4, 3]:
            item['title'] = 'kospi'
            item['date'] = response.xpath(f'/html/body/div/table[1]/tr[{i}]/td[1]/text()').get()
            item['value'] = response.xpath(f'/html/body/div/table[1]/tr[{i}]/td[2]/text()').get().replace(',', '')
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/sise/sise_index_day.nhn?code=KOSPI&page={page - 1}',
                callback=self.parse_kospi,
                cb_kwargs=dict(page=page - 1),
            )

    def parse_kosdaq(self, response, page):
        print(f"Parsing ...kosdaq {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in [12, 11, 10, 5, 4, 3]:
            item['title'] = 'kosdaq'
            item['date'] = response.xpath(f'/html/body/div/table[1]/tr[{i}]/td[1]/text()').get()
            item['value'] = response.xpath(f'/html/body/div/table[1]/tr[{i}]/td[2]/text()').get().replace(',', '')
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/sise/sise_index_day.nhn?code=KOSDAQ&page={page - 1}',
                callback=self.parse_kosdaq,
                cb_kwargs=dict(page=page - 1),
            )

    def parse_gbond3y(self, response, page):
        print(f"Parsing ...gbond3y {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(7, 0, -1):
            item['title'] = 'gbond3y'
            item['date'] = (response.css(f'body > div > table > tbody > tr:nth-child({i}) > td.date::text')
                            .extract()[0].replace('\n', '').replace('\t', ''))
            item['value'] = (response.css(f'body > div > table > tbody > tr:nth-child({i}) > td:nth-child(2)::text')
                .extract()[0])
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/interestDailyQuote.nhn?marketindexCd=IRR_GOVT03Y&page={page - 1}',
                callback=self.parse_gbond3y,
                cb_kwargs=dict(page=page - 1),
                )

    def parse_aud(self, response, page):
        print(f"Parsing ...aud {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(7, 0, -1):
            item['title'] = 'aud'
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?fdtc=4&marketindexCd=FX_USDAUD&page={page - 1}',
                callback=self.parse_aud,
                cb_kwargs=dict(page=page - 1),
                )

    def parse_chf(self, response, page):
        print(f"Parsing ...chf {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(7, 0, -1):
            item['title'] = 'chf'
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?fdtc=4&marketindexCd=FX_USDCHF&page={page - 1}',
                callback=self.parse_chf,
                cb_kwargs=dict(page=page - 1),
                )

    def parse_usdkrw(self, response, page):
        print(f"Parsing ...usdkrw {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(10, 0, -1):
            item['title'] = 'usdkrw'
            item['date'] = response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[1]/text()').get()
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[2]/text()')
                             .get().replace(',', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/exchangeDailyQuote.nhn?marketindexCd=FX_USDKRW&page={page - 1}',
                callback=self.parse_usdkrw,
                cb_kwargs=dict(page=page - 1),
            )

    def parse_wti(self, response, page):
        print(f"Parsing ...wti {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(7, 0, -1):
            item['title'] = 'wti'
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=OIL_CL&fdtc=2&page={page - 1}',
                callback=self.parse_wti,
                cb_kwargs=dict(page=page - 1),
            )

    def parse_gold(self, response, page):
        print(f"Parsing ...gold {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(7, 0, -1):
            item['title'] = 'gold'
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=CMDT_GC&fdtc=2&page={page - 1}',
                callback=self.parse_gold,
                cb_kwargs=dict(page=page - 1),
            )

    def parse_silver(self, response, page):
        print(f"Parsing ...silver {page} page")
        item = items.MisItems()
        # print(response.text)

        time.sleep(random.choice(rnd_wait))

        for i in range(7, 0, -1):
            item['title'] = 'silver'
            item['date'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[1]/text()')
                            .get().replace('\t', '').replace('\n', ''))
            item['value'] = (response.xpath(f'/html/body/div/table/tbody/tr[{i}]/td[2]/text()')
                             .get().replace(',', '').replace('\t', '').replace('\n', ''))
            print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
            yield item
        if page > 1:
            yield scrapy.Request(
                url=f'https://finance.naver.com/marketindex/worldDailyQuote.nhn?marketindexCd=CMDT_SI&fdtc=2&page={page - 1}',
                callback=self.parse_silver,
                cb_kwargs=dict(page=page - 1),
            )

    def parse_sp500(self, response, page):
        def yield_data():
            sel = Selector(text=self.webdriver.page_source)
            for i in range(10, 0, -1):
                item['title'] = 'sp500'
                item['date'] = sel.xpath(f'//*[@id="dayTable"]/tbody/tr[{i}]/td[1]/text()').get()
                item['value'] = (sel.xpath(f'//*[@id="dayTable"]/tbody/tr[{i}]/td[2]/span/text()')
                                 .get().replace(',', ''))
                print(f"title: {item['title']}, date : {item['date']}, value : {item['value']}")
                yield item

        print(f"Parsing ...sp500 get {page} tables from ({response.url}) ")
        self.webdriver.get(response.url)
        self.webdriver.implicitly_wait(3)
        item = items.MisItems()
        # print(response.text)

        # last_page_sp500은 일별시세 테이블을 몇번째까지 바꿀것인지를 뜻함.
        for i in range(1, self.last_page_sp500 + 1):
            time.sleep(random.choice(rnd_wait)-2)
            num_btn = f'//*[@id="dayLink{i}"]'
            try:
                self.webdriver.find_element(By.XPATH, num_btn).click()
            except NoSuchElementException:
                # 현재 페이지에 누를 수 있는 버튼이 없는경우.. 다음 버튼을 누른다.
                next_btn = '//*[@id="dayPaging"]/a[11]'
                self.webdriver.find_element(By.XPATH, next_btn).click()
                print(f"Click the next button...")
                time.sleep(random.choice(rnd_wait)-2)
                self.webdriver.find_element(By.XPATH, num_btn).click()

            print(f"Click the {num_btn} button...")
            time.sleep(random.choice(rnd_wait))
            # 선택한 페이지의 dayTable에서 데이터 추출함.
            yield from yield_data()



import os
from scrapy.crawler import CrawlerProcess
from webdriver_hj3415 import drivers

from scraper_hj3415.miscraper.mis.spiders.mi import MiSpider
from scraper_hj3415.miscraper.mis.spiders.mihistory import MIHistory

# 세팅파일을 프로젝트의 settings.py를 사용하지 않고 직접 만들어서 사용하는 방법.
# 대신 nfs 모듈을 찾을수 있도록 sys에 경로를 추가해 줘야 한다.
import sys
# Scrapy 프로젝트 경로 추가
sys.path.append(os.path.dirname(os.path.abspath(__file__)))

settings = {
'BOT_NAME': "mis",

'SPIDER_MODULES': ["mis.spiders"],
'NEWSPIDER_MODULE': "mis.spiders",

'ROBOTSTXT_OBEY': False,
'ITEM_PIPELINES': {
    "mis.pipelines.ValidationPipeline": 300,
    "mis.pipelines.RedisPipeline": 400,
    "mis.pipelines.MongoPipeline": 500,
},
'TWISTED_REACTOR': "twisted.internet.asyncioreactor.AsyncioSelectorReactor",
'FEED_EXPORT_ENCODING': "utf-8",

'LOG_ENABLED': True,
'LOG_LEVEL' : 'WARNING',
}


def mi():
    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(MiSpider)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림


def mihistory(years: int):
    from .. import headless, driver_version, browser
    webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(MIHistory, webdriver=webdriver, years=years)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    print('Retrieve webdriver...')
    webdriver.quit()






import os
from scrapy.crawler import CrawlerProcess
from webdriver_hj3415 import drivers

from scraper_hj3415.nfscraper.nfs.spiders.c101 import C101Spider
from scraper_hj3415.nfscraper.nfs.spiders.c106 import C106Spider
from scraper_hj3415.nfscraper.nfs.spiders.c103 import C103YSpider, C103QSpider
from scraper_hj3415.nfscraper.nfs.spiders.c104 import C104YSpider, C104QSpider
from scraper_hj3415.nfscraper.nfs.spiders.c108 import C108Spider

# 세팅파일을 프로젝트의 settings.py를 사용하지 않고 직접 만들어서 사용하는 방법.
# 대신 nfs 모듈을 찾을수 있도록 sys에 경로를 추가해 줘야 한다.
import sys
# Scrapy 프로젝트 경로 추가
sys.path.append(os.path.dirname(os.path.abspath(__file__)))

settings = {
'BOT_NAME': "nfs",

'SPIDER_MODULES': ["nfs.spiders"],
'NEWSPIDER_MODULE': "nfs.spiders",


'ROBOTSTXT_OBEY': False,
'ITEM_PIPELINES': {
    "nfs.pipelines.ValidationPipeline": 300,
    "nfs.pipelines.RedisPipeline": 400,
    "nfs.pipelines.MongoPipeline": 500,
},
'TWISTED_REACTOR': "twisted.internet.asyncioreactor.AsyncioSelectorReactor",
'FEED_EXPORT_ENCODING': "utf-8",

'LOG_ENABLED': True,
'LOG_LEVEL' : 'WARNING',
}

def c101(*args):
    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C101Spider, codes=args)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

def c106(*args):
    from .. import headless, driver_version, browser
    webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C106Spider, codes=args, webdriver=webdriver)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    print('Retrieve webdriver...')
    webdriver.quit()

def c103y(*args):
    from .. import headless, driver_version, browser
    webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C103YSpider, codes=args, webdriver=webdriver)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    print('Retrieve webdriver...')
    webdriver.quit()

def c103q(*args):
    from .. import headless, driver_version, browser
    webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C103QSpider, codes=args, webdriver=webdriver)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    print('Retrieve webdriver...')
    webdriver.quit()

def c104y(*args):
    from .. import headless, driver_version, browser
    webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C104YSpider, codes=args, webdriver=webdriver)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    print('Retrieve webdriver...')
    webdriver.quit()

def c104q(*args):
    from .. import headless, driver_version, browser
    webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C104QSpider, codes=args, webdriver=webdriver)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    print('Retrieve webdriver...')
    webdriver.quit()

def c108(*args):
    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    process.crawl(C108Spider, codes=args)
    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

def all_spider(*args):
    from .. import headless, driver_version, browser
    spiders = (C101Spider, C106Spider, C103YSpider, C103QSpider, C104YSpider, C104QSpider, C108Spider)
    wedrivers = []

    # Scrapy 설정 가져오기
    # settings = get_project_settings()
    # CrawlerProcess 인스턴스 생성
    process = CrawlerProcess(settings)
    # 스파이더 추가 및 실행
    for Spider in spiders:
        if Spider == C101Spider or Spider == C108Spider:
            process.crawl(Spider, codes=args)
        else:
            webdriver = drivers.get(browser=browser, driver_version=driver_version, headless=headless)
            process.crawl(Spider, codes=args, webdriver=webdriver)
            wedrivers.append(webdriver)

    process.start()  # 블로킹 호출, 스파이더가 완료될 때까지 기다림

    for webdriver in wedrivers:
        print('Retrieve webdriver...')
        webdriver.quit()



import time
import scrapy
import pandas as pd

from scraper_hj3415.nfscraper.nfs import items


class C108Spider(scrapy.Spider):
    name = 'c108'
    allowed_domains = ['navercomp.wisereport.co.kr']    # https 주소

    def __init__(self, *args, **kwargs):
        super(C108Spider, self).__init__(*args, **kwargs)
        self.codes = kwargs.get("codes", [])

    def start_requests(self):
        total_count = len(self.codes)
        self.logger.info(f'entire codes list - {self.codes}')
        for i, one_code in enumerate(self.codes):
            # reference from https://docs.scrapy.org/en/latest/topics/request-response.html
            yield scrapy.Request(
                url=f'https://navercomp.wisereport.co.kr/v2/company/c1080001.aspx?cmp_cd={one_code}',
                callback=self.parse_c108,
                cb_kwargs=dict(code=one_code)
            )

    def parse_c108(self, response, code):
        """
            C108 분석한 XPATH
            date_XPATH = f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[1]'               # 날짜
            content_XPATH = f'//*[@id="td{i}"]/@data-content'                               # 제목과내용
            author_XPATH = f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[3]/div'         # 작성자
            company_XPATH = f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[4]/div'        # 제공처
            opinion_XPATH = f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[5]/div[1]'     # 투자의견
            hprice_XPATH = f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[6]/div[1]'      # 목표가
        """
        time.sleep(2)
        print(f'<<< Parsing {self.name}...{code}')
        # c108페이지가 아직 만들어진 경우 빈테이블을 만들어 저장한다.
        title = response.xpath('/html/head/title/text()').get()
        if title == '기업재무정보 접속장애':
            self.logger.warning(f'{code}: None C108 data...We will make empty table..')
            # make item to yield
            item = items.C103468items()
            item['code'] = code
            item['page'] = 'c108'
            item['df'] = pd.DataFrame(columns=['날짜', '제목', '작성자', '제공처', '투자의견', '목표가', '내용'])
            yield item
            return

        # reference from http://hleecaster.com/python-pandas-creating-and-loading-dataframes/(데이터프레임 만들기)
        opinions = []
        # 표의 아이템 갯수는 최대 20개
        for i in range(20):
            one_opinion = list()
            # 1. 날짜추가
            one_opinion.append(response.xpath(f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[1]/text()').get())
            # 제목과 내용을 추출한 리스트에서 제목따로 내용따로 추출
            try:
                title_and_contents = response.xpath(f'//*[@id="td{i}"]/@data-content').get()\
                    .replace("<br/><span class='comment_text'>▶</span>", '▶').split('\n')
                self.logger.debug(title_and_contents)
            except AttributeError:
                break
            # 2. 제목을 추출하고 리스트에서 지움
            one_opinion.append(title_and_contents[0])
            del title_and_contents[0]
            # 3. 작성자, 제공처, 투자의견, 목표가를 차례로 추출하고 리스트에 추가
            for j in range(3, 7):
                one_opinion.append(response.xpath(f'//*[@id="tableCmpDetail"]/tr[{(i * 2) + 1}]/td[{j}]/div/text()')
                                   .get().replace('\t', '').replace('\r\n', ''))
            # 4. 내용을 추출하고 개행문자를 첨가함
            contents = ''
            for item in title_and_contents:
                contents += item
            one_opinion.append(contents)
            self.logger.debug(one_opinion)
            # 5. 완성된 row를 opinions리스트에 넣는다.
            opinions.append(one_opinion)
        self.logger.info(f'\ttotal {len(opinions)} opinions.. {code}')

        df = pd.DataFrame(data=opinions, columns=['날짜', '제목', '작성자', '제공처', '투자의견', '목표가', '내용'])
        df['내용'] = df['내용'].str.replace('\r', '')

        self.logger.debug(df)
        # make item to yield
        item = items.C103468items()
        item['code'] = code
        item['page'] = 'c108'
        item['df'] = df
        yield item



import os
import time
import shutil
from typing import Optional, List
import pandas as pd
import sqlite3

import requests
from datetime import datetime, timedelta

from scraper_hj3415 import nfs
from utils_hj3415 import setup_logger

mylogger = setup_logger(__name__,'WARNING')

sqlite_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'krx.db')
table_name = 'krx300'

def find_valid_url() -> str:
    delta=0
    while True:
        date = datetime.now() - timedelta(days=delta)
        date_str = date.strftime('%Y%m%d')
        url = f'https://www.samsungfund.com/excel_pdf.do?fId=2ETFA4&gijunYMD={date_str}'
        response = requests.get(url)
        time.sleep(3)
        if response.headers.get('Content-Length','0') != '0':
            print('삼성자산운용 krx300 엑셀다운 url : ', url)
            return url
        else:
            mylogger.warning(f'https://www.samsungfund.com/excel_pdf.do?fId=2ETFA4&gijunYMD={date_str} - 엑셀파일 다운에러')
            delta += 1

def download_exel() -> Optional[str]:
    TEMP_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'temp')

    # 임시폴더 정리
    shutil.rmtree(TEMP_DIR, ignore_errors=True)
    os.makedirs(TEMP_DIR, exist_ok=True)
    print(f"임시폴더 초기화 완료: {TEMP_DIR}")

    url = find_valid_url()
    response = requests.get(url)

    SAVE_PATH = os.path.join(TEMP_DIR, f'{url[-8:]}.xls')

    # 정상 응답인지 확인
    if response.status_code == 200:
        # 바이너리 모드('wb')로 파일 쓰기
        with open(SAVE_PATH, 'wb') as f:
            f.write(response.content)
        print(f"파일 다운로드 완료: {SAVE_PATH}")
        return SAVE_PATH
    else:
        print(f"다운로드 실패: 상태코드 {response.status_code}")
        return None

def make_db(excel_path: str):
    # 1. 엑셀 파일 읽기
    try:
        df = pd.read_excel(excel_path, usecols='B:I', skiprows=2)  # 첫 번째 시트를 읽음
    except Exception as e:
        print(f"엑셀 파일 읽기 실패: {e}")
        return

    # 2. SQLite 데이터베이스 연결
    try:
        conn = sqlite3.connect(sqlite_path)  # 데이터베이스 파일 생성/연결
        print(f"SQLite 데이터베이스 생성/연결 성공: {sqlite_path}")
    except Exception as e:
        print(f"SQLite 연결 실패: {e}")
        return

    # 3. 데이터베이스 테이블로 데이터 삽입
    try:
        df.to_sql(table_name, conn, if_exists='replace', index=False)  # 데이터 삽입
        print(f"데이터가 테이블 '{table_name}'에 성공적으로 삽입되었습니다.")
    except Exception as e:
        print(f"데이터 삽입 실패: {e}")

    # 4. 원화예금 항목을 삭제한다.
    try:
        cursor = conn.cursor()
        sql = 'DELETE FROM "krx300" WHERE "종목명" = ?'
        cursor.execute(sql, ('원화예금',))
        conn.commit()
        print(f"원화예금 항목 삭제")
    except sqlite3.Error as e:
        print(f"SQLite error occurred: {e}")
    finally:
        if conn:
            conn.close()  # 연결 종료
            print("SQLite 연결 종료.")

def get_codes() -> list:
    """
    Retrieve a list of stock codes from a specified SQLite database table.

    This function checks for the existence of an SQLite database file at the
    specified path. If the file does not exist, it logs a warning message,
    downloads a necessary Excel file, and creates the database file.
    It then queries the database to fetch stock codes meeting a specific pattern
    and returns them as a list.

    Returns:
        list: A list of stock code strings retrieved from the database.
    """
    if not os.path.exists(sqlite_path):
        mylogger.warning(f"{sqlite_path} 파일이 존재하지 않아 새로 생성합니다.")
        excel_path = download_exel()
        make_db(excel_path)
    with sqlite3.connect(sqlite_path) as conn:
        # 종목코드를 가져오는 쿼리
        query = f"SELECT 종목코드 FROM {table_name} WHERE 종목코드 LIKE '______'"
        codes = pd.read_sql(query, conn)['종목코드'].tolist()
    return codes

def get_code_names() -> List[list]:
    if not os.path.exists(sqlite_path):
        mylogger.warning(f"{sqlite_path} 파일이 존재하지 않아 새로 생성합니다.")
        excel_path = download_exel()
        make_db(excel_path)
    with sqlite3.connect(sqlite_path) as conn:
        # 종목코드와 종목명을 가져오는 쿼리
        query = f"SELECT 종목코드, 종목명 FROM {table_name} WHERE 종목코드 LIKE '______'"
        code_names = pd.read_sql(query, conn).values.tolist()
    return code_names

# 종목명으로 종목코드를 찾는 함수
def get_name(code: str):
    for code_sql, name_sql in get_code_names():
        if code == code_sql:
            return name_sql
    return None  # 종목명을 찾지 못한 경우 None 반환

def sync_with_mongo():
    # krx300 sqlite3 리프레시
    make_db(download_exel())

    from db_hj3415 import mymongo
    in_mongo_codes = mymongo.Corps.list_all_codes()
    in_sqlite_codes = get_codes()
    mylogger.info(f"In mongodb: {len(in_mongo_codes)} - {in_mongo_codes}")
    mylogger.info(f"In sqlite3: {len(in_sqlite_codes)} - {in_sqlite_codes}")

    del_difference = list(set(in_mongo_codes) - set(in_sqlite_codes))
    add_difference = list(set(in_sqlite_codes) - set( in_mongo_codes))

    if len(add_difference) == 0 and len(del_difference) == 0:
        print(f"mongodb와 krx300의 sync는 일치합니다.(총 {len(in_mongo_codes)} 종목)")
    else:
        print(f"mongodb에서 삭제될 코드: {len(del_difference)} - {del_difference}")
        print(f"mongodb에 추가될 코드: {len(add_difference)} - {add_difference}")

        # 몽고디비에서 불필요한 종목 삭제하고 서버에 기록.
        for code in del_difference:
            mymongo.Logs.save('mongo', 'WARNING', f'{code}/{mymongo.Corps.get_name(code)}를 삭제')
            mymongo.Corps.drop_code(code)

        # 몽고디비에 새로운 종목 추가하고 서버에 기록.
        if len(add_difference) != 0:
            nfs.all_spider(*add_difference)
            for code in add_difference:
                mymongo.Logs.save('mongo', 'WARNING', f'{code}/{get_name(code)}을 추가')



import scrapy
from scrapy.selector import Selector
from selenium.webdriver.remote.webdriver import WebDriver

from scraper_hj3415.nfscraper.nfs import items
from scraper_hj3415.nfscraper import common

'''
# XPATH 상수
수익성 = '//*[ @id="val_tab1"]'
성장성 = '//*[ @id="val_tab2"]'
안정성 = '//*[ @id="val_tab3"]'
활동성 = '//*[ @id="val_tab4"]'

연간 = '//*[@id="frqTyp0"]'
분기 = '//*[@id="frqTyp1"]'
검색 = '//*[@id="hfinGubun"]'

가치분석연간 = '//*[@id="frqTyp0_2"]'
가치분석분기 = '//*[@id="frqTyp1_2"]'
가치분석검색 = '//*[@id="hfinGubun2"]'
'''


class C104(scrapy.Spider):
    name = 'c104'
    allowed_domains = ['navercomp.wisereport.co.kr']

    # 버튼 클릭시 페이지가 변하는 것이 일관되지 않는것을 해결하기 위한 세팅. 챗gpt 검색
    custom_settings = {
        'AUTOTHROTTLE_ENABLED': True,
        'DOWNLOAD_DELAY': 1,
        'CONCURRENT_REQUESTS': 1,
        'CONCURRENT_REQUESTS_PER_DOMAIN': 1,
        'CONCURRENT_REQUESTS_PER_IP': 1,
    }

    def __init__(self, *args, **kwargs):
        super(C104, self).__init__(*args, **kwargs)
        self.codes = kwargs.get("codes", [])
        self.webdriver = kwargs.get("webdriver", None)

    @staticmethod
    def make_df(webdriver: WebDriver, code: str, page: str, title: str):
        # html에서 table을 추출하여 dataframe생성
        html = Selector(text=webdriver.page_source)
        table_xpath = '//table[@class="gHead01 all-width data-list"]'

        # 테이블명을 _을 기준으로 나눠 리스트를 만든다.
        title_list = title.split('_')

        # dataframe 리스트를 만든다.
        df_list = []
        for i in range(2):
            # 상위테이블 0, 하위테이블 1
            df_list.append(common.get_df_from_html(html, table_xpath, i))
        # print(df_list)

        # 테이블명리스트와 df리스트를 매치하여 데이터베이스에 저장하기 위해 yield시킴
        for _, df in list(zip(title_list, df_list)):
            # print(df)
            # make item to yield
            item = items.C103468items()
            item['code'] = code
            item['page'] = page
            item['df'] = df
            yield item

    def start_requests(self):
        # reference from https://docs.scrapy.org/en/latest/topics/request-response.html
        total_count = len(self.codes)
        print(f'Start scraping {self.name}, {total_count} items...')
        self.logger.info(f'entire codes list - {self.codes}')

        # 실제로 페이지를 스크랩하기위해 호출
        for i, one_code in enumerate(self.codes):
            yield scrapy.Request(
                url=f'https://navercomp.wisereport.co.kr/v2/company/c1040001.aspx?cmp_cd={one_code}',
                callback=getattr(self, 'parse_c104'),
                cb_kwargs=dict(code=one_code)
            )


class C104YSpider(C104):
    name = 'c104y'
    # 순서를 바꾸지 말것
    btns = {
        '수익성y_가치분석y': [
            ('수익성', '//*[ @id="val_tab1"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
            ('가치분석연간', '//*[@id="frqTyp0_2"]'),
            # ('가치분석검색', '//*[@id="hfinGubun2"]'),
        ],
        '성장성y': [
            ('성장성', '//*[ @id="val_tab2"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        '안정성y': [
            ('안정성', '//*[ @id="val_tab3"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        '활동성y': [
            ('활동성', '//*[ @id="val_tab4"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
    }

    def __init__(self, *args, **kwargs):
        super(C104YSpider, self).__init__(*args, **kwargs)

    def parse_c104(self, response, code):
        print(f'<<< Parsing C104Y ...{code}')
        self.webdriver.get(response.url)
        self.webdriver.implicitly_wait(10)
        for title, btn_list in C104YSpider.btns.items():
            print(f"Making dataframe - {code} / c104y {title}..")
            common.click_buttons(self.webdriver, btn_list)
            yield from C104.make_df(self.webdriver, code, 'c104y', title)


class C104QSpider(C104):
    name = 'c104q'

    # 순서를 바꾸지 말것
    btns = {
        '수익성q_가치분석q': [
            ('수익성', '//*[ @id="val_tab1"]'),
            ('분기', '//*[@id="frqTyp1"]'),
            ('검색', '//*[@id="hfinGubun"]'),
            ('가치분석분기', '//*[@id="frqTyp1_2"]'),
            ('가치분석검색', '//*[@id="hfinGubun2"]'),
        ],
        '성장성q': [
            ('성장성', '//*[ @id="val_tab2"]'),
            # ('분기', '//*[@id="frqTyp1"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        '안정성q': [
            ('안정성', '//*[ @id="val_tab3"]'),
            # ('분기', '//*[@id="frqTyp1"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        '활동성q': [
            ('활동성', '//*[ @id="val_tab4"]'),
            # ('분기', '//*[@id="frqTyp1"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
    }

    def __init__(self, *args, **kwargs):
        super(C104QSpider, self).__init__(*args, **kwargs)

    def parse_c104(self, response, code):
        print(f'<<< Parsing C104Q ...{code}')
        self.webdriver.get(response.url)
        self.webdriver.implicitly_wait(10)
        for title, btn_list in C104QSpider.btns.items():
            print(f"Making dataframe - {code} / c104q {title}..")
            common.click_buttons(self.webdriver, btn_list)
            yield from C104.make_df(self.webdriver, code, 'c104q', title)











import scrapy
from scrapy.selector import Selector
from selenium.webdriver.remote.webdriver import WebDriver

from scraper_hj3415.nfscraper.nfs import items
from scraper_hj3415.nfscraper import common


'''
# XPATH 상수
손익계산서 = '//*[@id="rpt_tab1"]'
재무상태표 = '//*[@id="rpt_tab2"]'
현금흐름표 = '//*[@id="rpt_tab3"]'
연간 = '//*[@id="frqTyp0"]'
분기 = '//*[@id="frqTyp1"]'
검색 = '//*[@id="hfinGubun"]'
'''


class C103(scrapy.Spider):
    allowed_domains = ['navercomp.wisereport.co.kr']

    # 버튼 클릭시 페이지가 변하는 것이 일관되지 않는것을 해결하기 위한 세팅. 챗gpt 검색
    custom_settings = {
        'AUTOTHROTTLE_ENABLED': True,
        'DOWNLOAD_DELAY': 1,
        'CONCURRENT_REQUESTS': 1,
        'CONCURRENT_REQUESTS_PER_DOMAIN': 1,
        'CONCURRENT_REQUESTS_PER_IP': 1,
    }

    def __init__(self, *args, **kwargs):
        super(C103, self).__init__(*args, **kwargs)
        self.codes = kwargs.get("codes", [])
        self.webdriver = kwargs.get("webdriver", None)

    @staticmethod
    def make_df(webdriver: WebDriver, code: str, page: str):
        # html에서 table을 추출하여 dataframe생성
        html = Selector(text=webdriver.page_source)
        table_xpath = '//table[2]'

        df = common.get_df_from_html(html, table_xpath, 1)
        # print(df)

        # make item to yield
        item = items.C103468items()
        item['code'] = code
        item['page'] = page
        item['df'] = df
        yield item

    def start_requests(self):
        # reference from https://docs.scrapy.org/en/latest/topics/request-response.html
        total_count = len(self.codes)
        print(f'Start scraping {self.name}, {total_count} items...')
        self.logger.info(f'entire codes list - {self.codes}')

        # 실제로 페이지를 스크랩하기위해 호출
        for i, one_code in enumerate(self.codes):
            yield scrapy.Request(
                url=f'https://navercomp.wisereport.co.kr/v2/company/c1030001.aspx?cmp_cd={one_code}',
                callback=getattr(self, 'parse_c103'),
                cb_kwargs=dict(code=one_code)
            )


class C103YSpider(C103):
    name = 'c103y'

    # 순서 바꾸지 말것
    btns = {
        "손익계산서y": [
            ('손익계산서', '//*[@id="rpt_tab1"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        "재무상태표y": [
            ('재무상태표', '//*[@id="rpt_tab2"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        "현금흐름표y": [
            ('현금흐름표', '//*[@id="rpt_tab3"]'),
            # ('연간', '//*[@id="frqTyp0"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
    }

    def __init__(self, *args, **kwargs):
        super(C103YSpider, self).__init__(*args, **kwargs)

    def parse_c103(self, response, code):
        print(f'<<< Parsing C103Y ...{code}')
        self.webdriver.get(response.url)
        self.webdriver.implicitly_wait(10)
        for title, btn_list in C103YSpider.btns.items():
            page = self.name[:-1] + title
            print(f"Making dataframe - {code} / {page}..")
            common.click_buttons(self.webdriver, btn_list)
            yield from C103.make_df(self.webdriver, code, page)


class C103QSpider(C103):
    name = 'c103q'

    # 순서 바꾸지 말것
    btns = {
        "손익계산서q": [
            ('손익계산서', '//*[@id="rpt_tab1"]'),
            ('분기', '//*[@id="frqTyp1"]'),
            ('검색', '//*[@id="hfinGubun"]'),
        ],
        "재무상태표q": [
            ('재무상태표', '//*[@id="rpt_tab2"]'),
            # ('분기', '//*[@id="frqTyp1"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
        "현금흐름표q": [
            ('현금흐름표', '//*[@id="rpt_tab3"]'),
            # ('분기', '//*[@id="frqTyp1"]'),
            # ('검색', '//*[@id="hfinGubun"]'),
        ],
    }

    def __init__(self, *args, **kwargs):
        super(C103QSpider, self).__init__(*args, **kwargs)

    def parse_c103(self, response, code):
        print(f'<<< Parsing C103Q ...{code}')
        self.webdriver.get(response.url)
        self.webdriver.implicitly_wait(10)
        for title, btn_list in C103QSpider.btns.items():
            page = self.name[:-1] + title
            print(f"Making dataframe - {code} / {page}..")
            common.click_buttons(self.webdriver, btn_list)
            yield from C103.make_df(self.webdriver, code, page)



import yfinance as yf

# 금 선물 티커: GC=F
gold_ticker = yf.Ticker("GC=F")

# 최근 1년간 일별 금 시세 데이터 가져오기
gold_data = gold_ticker.history(period="1y")

# 데이터 출력
print(gold_data)
print(gold_data.columns)

# 'Close' 컬럼만 선택
close_data = gold_data['Close']
print(close_data)

# 금에 대한 기본 정보 및 통계
gold_info = gold_ticker.info
print(gold_info)


# Define your item pipelines here
#
# Don't forget to add your pipeline to the ITEM_PIPELINES setting
# See: https://docs.scrapy.org/en/latest/topics/item-pipeline.html

from db_hj3415 import mymongo, myredis


class ValidationPipeline:
    def process_item(self, item, spider):
        return item


class RedisPipeline:
    def process_item(self, item, spider):
        print(f"\tIn RedisPipeline.. db: mi / index: {item['title']} / date: {item['date']} / value: {item['value']}")
        mi_index = 'mi.' + item['title']
        del_redis_names = [ mi_index + '_recent', mi_index + '_trend']
        for redis_name in del_redis_names:
            print(f"\t\tDelete redis name : '{redis_name}'")
            myredis.Base.delete(redis_name)
        return item


class MongoPipeline:
    # 몽고 데이터 베이스에 저장하는 파이프라인
    def process_item(self, item, spider):
        """
        아이템 구조
            title = scrapy.Field()
            date = scrapy.Field()
            value = scrapy.Field()
        """
        print(f"\tIn MongoPipeline.. db: mi / index: {item['title']} / date: {item['date']} / value: {item['value']}")
        mymongo.MI.save(item['title'], {"date": item['date'], "value": item['value']})
        return item


import time
import pandas as pd
from typing import Tuple, List
from io import StringIO
import random

from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException

from utils_hj3415.logger import setup_logger

mylogger = setup_logger(__name__,'WARNING')


# 2초 이내는 데이터 캡쳐가 올바르게 안됨
rnd_wait = [3, 4, 5]


def click_buttons(driver: WebDriver, buttons: List[Tuple[str, str]]) -> bool:
    """
    하부 클래스에서 buttons 리스트를 입력받아 실제 버튼을 클릭하는 함수
    :return: 함수 작업이 무사히 완료되면 True
    """
    mylogger.debug(f'*** Setting page by clicking buttons ***')
    # print(buttons)
    for name, xpath in buttons:
        # print(f'- Click the {name} / {xpath} button')
        try:
            # 엘리먼트가 로드될 때까지 x초 대기
            element = WebDriverWait(driver, random.choice(rnd_wait) * 2).until(
                EC.element_to_be_clickable((By.XPATH, xpath))
            )
            # 엘리먼트 클릭
            # element.click()
            driver.execute_script("arguments[0].click();", element)
        except TimeoutException:
            print(f"{name} 엘리먼트를 찾지 못했습니다.")
            return False
        time.sleep(random.choice(rnd_wait))
    mylogger.debug('*** Buttons click done ***')
    return True


def get_df_from_html(selector, xpath, table_num):
    """
    C103,C104에서 사용
    펼치지 않은 네이버 테이블의 항목과 내용을 pandas 데이터프레임으로 변환시킴
    reference from http://hleecaster.com/python-pandas-selecting-data/(pandas 행열 선택)
    reference from https://blog.naver.com/wideeyed/221603778414(pandas 문자열 처리)
    reference from https://riptutorial.com/ko/pandas/example/5745/dataframe-%EC%97%B4-%EC%9D%B4%EB%A6%84-%EB%82%98%EC%97%B4(pandas 열이름 나열)
    """
    # 전체 html source에서 table 부위만 추출하여 데이터프레임으로 변환
    tables_list = []
    for table in selector.xpath(xpath).getall():
        tables_list.append(StringIO(table))

    df = pd.read_html(tables_list[table_num])[0]

    # 항목열의 펼치기 스트링 제거
    df['항목'] = df['항목'].str.replace('펼치기', '').str.strip()
    # reference from https://stackoverflow.com/questions/3446170/escape-string-for-use-in-javascript-regex(정규표현식 특수기호처리)
    # 인덱스행의 불필요한 스트링 제거
    df.columns = (df.columns.str.replace('연간컨센서스보기', '', regex=False)
                  .str.replace('연간컨센서스닫기', '', regex=False)
                  .str.replace('(IFRS연결)', '', regex=False)
                  .str.replace('(IFRS별도)', '', regex=False)
                  .str.replace('(GAAP개별)', '', regex=False)
                  .str.replace('(YoY)', '', regex=False)
                  .str.replace('(QoQ)', '', regex=False)
                  .str.replace('(E)', '', regex=False)
                  .str.replace('.', '', regex=False)
                  .str.strip())
    return df



from scraper_hj3415.nfscraper import run as nfs
from scraper_hj3415.miscraper import run as mis
from scraper_hj3415.krx300 import krx300 as krx

from dotenv import load_dotenv
from utils_hj3415.tools import get_env_path
from utils_hj3415.logger import mylogger

env_path = get_env_path()
if env_path is None:
    mylogger.warning(f"환경변수 파일(.env)를 찾을수 없습니다. 기본 설정값으로 프로그램을 실행합니다.")
load_dotenv(env_path)

import os
headless = os.getenv('HEADLESS', 'true').lower() in ["true", "1", "yes"]
driver_version = os.getenv('DRIVER_VERSION', '')
browser = os.getenv('BROWSER', 'chrome')